#include <avr/io.h>
#include <stdio.h>
#define F_CPU 16000000UL
#include <util/delay.h>
#include "lcd_ch.h"

#define LCD_CLR 0      // DB0: Vy?istenie displaja
#define LCD_CTRL_PORT DDRD
#define LCD_DATA_PORT DDRB

#define LCD_RS_pin 2	// Zadefinovanie pinov LCD displaja
#define LCD_RW_pin 3
#define LCD_EN_pin 4

#define LCD_D4_pin 1
#define LCD_D5_pin 2
#define LCD_D6_pin 3
#define LCD_D7_pin 4

#define S0_PIN  PD5		// Zadefinovanie pinov snma?a
#define S1_PIN  PD6
#define S2_PIN  PD7
#define S3_PIN  PB0
#define OUT_PIN  PB5

void setup() 
{
    //LCD:
	DDRD |= (1<<LCD_EN_pin);	// Pin D4 (Enable)  PORTD  vstup  
	DDRD |= (1<<LCD_RW_pin);	// Pin D3 (RW)      PORTD  vstup  
	DDRD |= (1<<LCD_RS_pin);	// Pin D2 (RS)      PORTD  vstup  
	
	LCD_DATA_PORT |= (1<<LCD_D4_pin)|(1<<LCD_D5_pin)|(1<<LCD_D6_pin)|(1<<LCD_D7_pin);	// Piny 1,2,3,4, PORTB ako vstupy (Dta pre display)
	
	//Snimac:
	DDRD |= (1 << S0_PIN) | (1 << S1_PIN) | (1 << S2_PIN);   // Nastavenie pinov S0, S1 a S2 ako vstupy
	DDRB |= (1 << S3_PIN);		// Nastavenie S3 ako vstup
    DDRB &= ~(1 << OUT_PIN);	// Nastavenie pinu OUT ako vstup

    // Nastavenie frekven?nej kly na 2%
    PORTD &= ~(1 << S0_PIN);   // S0: 0
    PORTD |= (1 << S1_PIN);    // S1: 1
}

uint16_t TCSMeasure()
{
   if(!(PINB & (1 << OUT_PIN)))		// 
   {
      while(!(PINB & (1 << OUT_PIN)));   //?akanie na nben hranu  
   }

   while(PINB & (1 << OUT_PIN));   //?akanie na doben hranu

   TCNT1=0x0000;	//Resetovanie po?tadla

   TCCR1B=(1<<CS10);	//Nastavenie predeli?ky = F_CPU/1 (za?iatok po?tania)

   while(!(PINB & (1 << OUT_PIN)));   //?akanie na nben hranu

   TCCR1B=0x00;		//Zastavenie po?tadla

   return ((float)8000000UL/TCNT1);

}

long map(long x, long in_min, long in_max, long out_min, long out_max)		//Funkcia map na kalibrciu snma?a
{
  return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;		//(premenn, najmenia dan hodnota, najv?ia dan hodnota, najmenia poadovan hodnota, najv?ia poadovan hodnota)
}


int main(void) 
{  
	setup();	// inicicia nastavenia vstupov
	
	lcd_init();  // inicicia LCD

	void lcd_puts(const char *s)	// funkcia na pouivanie puts
	{
		register unsigned char c;
		while((c = *s++))
			lcd_data(c);
	}	

	char vysledokR[  ]={  };	// nastavenie premennch na zapisovanie hodnt
	char vysledokG[  ]={  };
	char vysledokB[  ]={  };
		
    while(1)
	{
		lcd_command(1 <<  LCD_CLR);		// vy?istenie displaja
		lcd_command(0b00001100);	//odstranenie kurzora
		lcd_command(0x80);		// nastavenie na prv riadok
		lcd_puts("Namerana farba:");	//Napiseme do prveho riadku
		
		PORTD &= ~(1 << S2_PIN);	//Nastavenie ?ervenho filtra
		PORTB &= ~(1 << S3_PIN);

		unsigned int hodnotaR = TCSMeasure();	//Uloenie nameranej hodnoty pre ?erven filter
		hodnotaR = map(hodnotaR,433,2195,0,255);	//upravenie na poadovan hodnoty
		
		lcd_command(0xC0);		// nastavenie na druh riadok
		lcd_puts("R=");
        sprintf(vysledokR,"%d",hodnotaR);	// prepsanie ?sla ako charakter
		lcd_puts(vysledokR);		//Vypisanie na LCD displaj
		_delay_ms(10);
		
		PORTD |= (1 << S2_PIN);		//Nastavenie zelenho filtra
		PORTB |= (1 << S3_PIN);
		
		unsigned int hodnotaG = TCSMeasure();	//Uloenie nameranej hodnoty pre zelen filter
		hodnotaG = map(hodnotaG,384,1300,0,255);	//Upravenie na poadovan hodnoty
		
		lcd_command(0xC0+5);	// nastavenie na 6tu pozciu na druhom riadku
		lcd_puts("G=");
		sprintf(vysledokG,"%d",hodnotaG);	// prepsanie ?sla ako charakter
		lcd_puts(vysledokG);		//Vypisanie na LCD displaji
		_delay_ms(10);
		
		PORTD &= ~(1 << S2_PIN);	//Nastavenie modrho filtra
		PORTB |= (1 << S3_PIN);
		
		unsigned int hodnotaB = TCSMeasure();	//Uloenie nameranej hodnoty pre modr filter
		hodnotaB = map(hodnotaB,487,2100,0,255);	//Upravenie na poadovan hodtoty
		
		lcd_command(0xC0+10);	// Nastavenie na 11tu poziciu na druhom riadku
		lcd_puts("B=");
		sprintf(vysledokB,"%d",hodnotaB);	// prepsanie ?sla ako charakter
		lcd_puts(vysledokB);	//Vypisanie na LCD dispaji
		_delay_ms(200);

	}     
   
}